# Установка и запуск Qlad на Centos 7

Для запуска на Ubuntu 18.04 см. [USAGE-ubuntu.md](./USAGE-ubuntu.md).

## Установка

Распакуйте предоставленный архив в произвольную директорию на диске

```
tar -xf qlad-centos-1.0.0-rc.1.tar.gz
```

Все перечисленные ниже шаги выполняются от пользователя с root-правами
в директории `qlad-centos-1.0.0-rc.1`.

1. Установите репозиторий EPEL:

    ```
    yum -y install epel-release
    ```

1. Установите libsodium:

    ```
    yum install -y libsodium
    ```

1. Устновите Python3 и pip:

    ```
    yum install -y python3 python3-pip
    ```

1. Установите Protobuf:

    ```
    yum install -y wget emacs-filesystem zlib-devel
    wget https://cbs.centos.org/kojifiles/packages/protobuf/3.6.1/4.el7/x86_64/protobuf-3.6.1-4.el7.x86_64.rpm
    wget https://cbs.centos.org/kojifiles/packages/protobuf/3.6.1/4.el7/x86_64/protobuf-compiler-3.6.1-4.el7.x86_64.rpm
    wget https://cbs.centos.org/kojifiles/packages/protobuf/3.6.1/4.el7/x86_64/protobuf-devel-3.6.1-4.el7.x86_64.rpm
    rpm -i protobuf-3.6.1-4.el7.x86_64.rpm
    rpm -i protobuf-compiler-3.6.1-4.el7.x86_64.rpm
    rpm -i protobuf-devel-3.6.1-4.el7.x86_64.rpm
    ```

1. Установите Exonum Launcher:

    ```
    pip3 install exonum-launcher --upgrade --no-binary=protobuf
    ```

1. Установите Qlad Launcher plugin:

    ```
    pip3 install -e launcher_instance_plugin
    ```

1. Установите Qlad:

    ```
    rpm -i qlad*.rpm
    ```

1. Установите QHB, QCP и Metricsd:

    ```
    rpm --import \
        https://repo.granit-concern.ru/qhb/keys/RPM-GPG-KEY-qhb && \
    yum-config-manager --add-repo \
        https://repo.granit-concern.ru/qhb/std-1/centos/7/x86_64/qhb.repo
    yum install -y qcp metricsd qhb-core qhb-contrib

    mkdir -p /opt/qhb-data \
        && chown qhb:qhb /opt/qhb-data \
        && su - qhb -c "/usr/local/qhb/bin/initdb -D /opt/qhb-data --encoding=UTF8" \
        && echo "metrics_collector_id = 100500" >> /opt/qhb-data/qhb.conf \
        && echo "shared_preload_libraries = 'librbytea'" >> /opt/qhb-data/qhb.conf
    ```

1. Установите NodeJS:

    ```
    curl -o- https://raw.githubusercontent.com/nvm-sh/nvm/v0.35.3/install.sh | bash
    export NVM_DIR="$HOME/.nvm"
    [ -s "$NVM_DIR/nvm.sh" ] && \. "$NVM_DIR/nvm.sh"
    nvm install node
    ```

## Запуск

1. Запустите QHB, QCP и metricsd

    ```
    cp ./metricsd-config.yaml /etc/metricsd/config.yaml
    cp ./qcp-config.yaml /etc/qcp/config.yaml
    su - qhb -c "/usr/local/qhb/bin/qhb_ctl -D /opt/qhb-data -l /opt/qhb-data/logfile start"
    systemctl start metricsd
    systemctl start qcp
    ```

1. Инициализируйте базу

    ```
    cp *.sql /opt/qhb-data
    chown qhb:qhb /opt/qhb-data/init-files.sql /opt/qhb-data/init-logs.sql
    su - qhb -c "/usr/local/qhb/bin/psql -h localhost -p 8080 -f /opt/qhb-data/init-files.sql"
    su - qhb -c "/usr/local/qhb/bin/psql -h localhost -p 8080 -f /opt/qhb-data/init-logs.sql"
    ```

1. Сгенерируйте конфигурацию блокчейн-сети из 1 узла

    ```
    qlad generate-template config/template.toml --validators-count 1
    sed -i 's/propose_timeout_threshold = 500/propose_timeout_threshold = 1/' config/template.toml
    sed -i 's/max_propose_timeout = 200/max_propose_timeout = 600000/' config/template.toml
    sed -i 's/min_propose_timeout = 10/min_propose_timeout = 200/' config/template.toml
    sed -i 's/first_round_timeout = 3000/first_round_timeout = 1200000/' config/template.toml
    qlad generate-config config/template.toml config/1 --peer-address 127.0.0.1:7001 -n
    qlad finalize config/1/sec.toml config/1/node.toml --public-api-address 127.0.0.1:8201 --private-api-address 127.0.0.1:8301 --public-configs config/1/pub.toml
    ```

    **Замечание**: если указанные адреса `--public-api-address` и/или `--private-api-address`
    отличаются от приведенных в примере выше, следует также обновить содержимое файла `qlad.yaml`
    в корне распакованного архива - оно должно содержать корретные адреса узла в секции `network`.

1. Запустите узел блокчейна

    ```
    qlad run --node-config config/1/node.toml --db-path db/1 --master-key-pass pass --host "127.0.0.1" --port 8080 &
    ```

1. Инициализируйте блокчейн, используя Launcher:

    ```
    python3 -m exonum_launcher -i ./qlad.yaml
    ```

    **Замечание**. В конфигурационном файле, предоставленном по умолчанию, публичный и приватный
    адреса узла указаны как `http://localhost:8201` и `http://localhost:8301` соответственно.
    Обновите содержимое `qlad.yaml`, если хотите использовать другие адреса.

    По умолчанию, блокчейн инициализируется с одним админ-пользователем с приватным ключом
    `89f28ef7604f738d86067a1f01314fd2bc23be2383b509c6d8ca861da0b6de0ce3e2ba391445701dcbf1a96bf63bec18efc118b4b0c8f1fa3e38516e6c20e200`. Ключ можно изменить, отредактировав файл `qlad.yaml`.

1. Запустите фронтенд

    Фронтенду передаются два параметра:
     - `port`: внешний порт для пользователей фронтенда. После запуска, фронтенд будет доступен
       по адресу `http://localhost:<port>` в браузере.
     - `api-root`: публичный адрес узла блокчейна. Должен соответствовать адресу
       `--public-api-address`, переданному команде `qlad run` ранее.

    Оба этих адреса должны быть доступны для внешних пользователей, обновите настройки фаервола,
    если необходимо.

    ```
    cd frontend
    node server.js --port 2867 --api-root=http://localhost:8201
    ```

После этого фронтенд доступен по адресу `http://localhost:2867`.
