from typing import Any

from exonum_client.protobuf_loader import ProtobufLoader
from exonum_client.module_manager import ModuleManager

from exonum_launcher.configuration import Instance

from exonum_launcher.instances.instance_spec_loader import InstanceSpecLoader, InstanceSpecLoadError


class QladInstanceSpecLoader(InstanceSpecLoader):
    def load_spec(self, loader: ProtobufLoader, instance: Instance) -> bytes:
        return self.serialize_config(loader, instance, instance.config)

    def serialize_config(self, loader: ProtobufLoader, instance: Instance, config: Any) -> bytes:
        try:
            try:
                # Try to load module (if it's already compiled) first.
                service_module = ModuleManager.import_service_module(
                    instance.artifact.name, instance.artifact.version, "service"
                )
            except (ModuleNotFoundError, ImportError):
                # If it's not compiled, load & compile protobuf.
                loader.load_service_proto_files(
                    instance.artifact.runtime_id, instance.artifact.name, instance.artifact.version
                )
                service_module = ModuleManager.import_service_module(
                    instance.artifact.name, instance.artifact.version, "service"
                )

            config_msg = service_module.Config()
            for pk in config['admins']:
                config_msg.admins.add().data = bytes(bytearray.fromhex(pk))

            result = config_msg.SerializeToString()

        # We're catching all the exceptions to shutdown gracefully (on the caller side) just in case.
        # pylint: disable=broad-except
        except Exception as error:
            artifact_name = instance.artifact.name
            raise InstanceSpecLoadError(
                f"Couldn't get a proto description for artifact: {artifact_name}, error: {error}"
            )

        return result
